function buildme()
	man = {
		function() love.graphics.circle("line", sw*0.68, sh*0.37, sh*0.05) end,
		function() love.graphics.line(sw*0.68, sh*0.42, sw*0.68, sh*0.62) end,
		function() love.graphics.line(sw*0.68, sh*0.48, sw*0.60, sh*0.44) end,
		function() love.graphics.line(sw*0.68, sh*0.48, sw*0.76, sh*0.44) end,
		function() love.graphics.line(sw*0.68, sh*0.62, sw*0.60, sh*0.76) end,
		function()
			love.graphics.line(sw*0.68, sh*0.62, sw*0.76, sh*0.76) 
			
			love.graphics.arc("line", sw*0.69, sh*0.39, sh*0.01, 0, math.pi)
			love.graphics.line(sw*0.66, sh*0.39, sw*0.70, sh*0.39)
			
			love.graphics.line(sw*0.66, sh*0.36, sw*0.67, sh*0.35)
			love.graphics.line(sw*0.66, sh*0.35, sw*0.67, sh*0.36)
			
			love.graphics.line(sw*0.69, sh*0.36, sw*0.70, sh*0.35)
			love.graphics.line(sw*0.69, sh*0.35, sw*0.70, sh*0.36)
		end,
	}
end

function love.resize(w, h)
	sw = w
	sh = h
	buildme()
end

function love.load(arg)
	math.randomseed(os.time())
	math.random()

	loadwords()

  sw = love.graphics.getWidth();
  sh = love.graphics.getHeight();
	
	love.graphics.setFont(fonts.mono46)
	fh = fonts.mono46:getHeight()
	fw = fonts.mono46:getWidth('mm')

	buildme()
	newgame()
end

function newgame()
	bad = {}
	word = getwordoflength(math.random(6,14))
	found = string.rep("_", #word)
	lost = string.rep(" ", #word)
end

function love.keyreleased(key)
	if #bad >= #man or not found:find('_') then
		if key == " " then
			newgame()
		end
		return
	end
	if key:len() == 1 and
	key:byte() >= string.byte('a') and
	key:byte() <= string.byte('z') then
		local good = false
		for i in word:gmatch('()'..key) do
			found = found:gsub('().', {[i]=key})
			good = true
		end
		if not good and not bad[key] then
			table.insert(bad, key)
			bad[key] = true
		end
		if #bad >= #man then
			--game over
			for i in found:gmatch('()_') do
				lost = lost:sub(1,i-1)..word:sub(i,i)..lost:sub(i+1)
			end
		end
	end
end

function love.draw()
	love.graphics.print(found, sw*0.05, sh*0.05)
	love.graphics.print(lost, sw*0.05, sh*0.05)
	for i = 1, #bad do

		local h = math.floor((sh-sh*0.2-fh)/fh)
		
		local x = math.ceil(i/h)
		local y = (i-1)%h
		
		x = x*fw + sw*0.05-fw
		y = y*fh + sh*0.1+fh

		love.graphics.print(bad[i], x, y)

		if man[i] then man[i]() end
	end

  love.graphics.rectangle("fill", sw*0.64, sh*0.90, sw*0.30, sh*0.04)
  love.graphics.rectangle("fill", sw*0.82, sh*0.24, sw*0.05, sh*0.66)
  love.graphics.rectangle("fill", sw*0.67, sh*0.20, sw*0.20, sh*0.04)
  love.graphics.rectangle("fill", sw*0.67, sh*0.24, sw*0.02, sh*0.07)

	if not found:find('_') then
		love.graphics.arc("line", sw*0.68, sh*0.38, sh*0.03, 0, math.pi)
		love.graphics.arc("line", sw*0.66, sh*0.36, sh*0.01, 0, -math.pi)
		love.graphics.arc("line", sw*0.70, sh*0.36, sh*0.01, 0, -math.pi)
	end
end

function loadwords()
	local chars, size = love.filesystem.read('wordlist')

	charweights = {}
	for num = string.byte("a"), string.byte("z") do
		charweights[string.char(num)] = 0
	end

	totalweight = 0

	for i = 1, #chars do
		local char = chars:sub(i, i)
		if charweights[char] ~= nil then -- avoid whitespace
			charweights[char] = charweights[char] + 1
			totalweight = totalweight + 1
		end
	end

	lengths = {}
  
  words = {}
  local index = 1
  for word in string.gmatch(chars, "%S+") do
    words[word] = true
    words[index] = word
		
		length = word:len()
		if not lengths[length] then
			lengths[length] = {}
		end
		table.insert(lengths[length], index)
    
		index = index + 1
  end
	
	fonts = {
		mono46 = love.graphics.newFont("dejavu/DejaVuSansMono.ttf", 46),
		mono36 = love.graphics.newFont("dejavu/DejaVuSansMono.ttf", 36),
	}
end

-- http://snippets.luacode.org/?p=snippets/Weighted_random_choice_104
function weightedrandomchar()

  local threshold = math.random(0, totalweight)
  local lastchar

  for char, weight in pairs(charweights) do
    threshold = threshold - weight

    if threshold <= 0 then return char end
		lastchar = char
  end

  return lastchar
end

function getwordoflength(len)
	return words[lengths[len][math.random(1, #lengths[len])]]
end
